-- Copyright (c) 2015-present, Facebook, Inc.
-- All rights reserved.
--
-- This source code is licensed under the BSD-style license found in the
-- LICENSE file in the root directory of this source tree. An additional grant
-- of patent rights can be found in the PATENTS file in the same directory.


local class = require 'class'

local tablex = require 'pl.tablex'

local actions = require 'babi.actions'
local Task = require 'babi.Task'
local World = require 'babi.World'
local Question = require 'babi.Question'
local Clause = require 'babi.Clause'
local Entity = require 'babi.Entity'

local WhereIsActor = class('WhereIsActor', 'Task')

function WhereIsActor:new_world()
    local world = World()
    world:load((BABI_HOME or '') .. 'tasks/worlds/world_basic.txt')
    return world
end

function WhereIsActor:generate_story(world, knowledge, story)
    -- Find the actors and the locations in the world
    local actors    = world:get_actors()
    local locations = world:get_locations()
    local isIQA     = true;
    local sheEntity = Entity('she', {['is_actor']=true, ['is_female']=true});
    local heEntity  = Entity('he', {['is_actor']=true, ['is_male'] = true});
    
    -- Our story will be 2 statements, 1 question, 5 times
    local index       = 1;
    local firstActor  = nil;
    local females     = {};
    local males       = {};
    local femaleCount = 0;
    local maleCount   = 0;
    for i = 1, 15 do
        if i % 3 ~= 0 then
            -- Find a random action
            local clause = Clause.sample_valid(world, {true}, actors,
                                               {actions.teleport}, locations)
            if isIQA then
                if i == 1 then
                    firstActor = clause.actor;
                elseif i == 2 then
                    while clause.actor.name == firstActor.name or
                                clause.actor.is_male ~= firstActor.is_male do
                        clause = Clause.sample_valid(world, {true}, actors,
                                           {actions.teleport}, locations);
                    end
                end
                if clause.actor.is_male then
                    if not males[clause.actor.name] then
                        maleCount = maleCount + 1;
                        males[clause.actor.name] = true;
                    end
                else
                    if not females[clause.actor.name] then
                        femaleCount = femaleCount + 1;
                        females[clause.actor.name] = true;
                    end
                end
            end
            clause:perform()
            story[index] = clause
            knowledge:update(clause)
        else
            -- Find the actors of which we know the location
            local known_actors = tablex.filter(
                knowledge:current():find('is_in'),
                function(entity) return entity.is_actor end
            )

            -- Pick a random one and ask where he/she is
            local random_actor = known_actors[math.random(#known_actors)]
            local value, support =
                knowledge:current()[random_actor]:get_value('is_in', true)

            if isIQA then
                while (random_actor.is_male and maleCount == 1) or
                        (random_actor.is_female and femaleCount == 1)  do
                    random_actor = known_actors[math.random(#known_actors)]
                    value, support =
                        knowledge:current()[random_actor]:get_value('is_in', true)
                end

                local askRef = random_actor.is_male == true and heEntity or sheEntity;
                story[index] = Question(
                    'eval',
                    Clause(world, true, world:god(), actions.set,
                           askRef, 'is_in', value),
                    support
                )
                index = index + 1;
                story[index] = Question(
                    'complementaryRef',
                    Clause(world, true, world:god(), actions.set,
                           askRef, random_actor),
                    support
                )
            else
                story[index] = Question(
                    'eval',
                    Clause(world, true, world:god(), actions.set,
                           random_actor, 'is_in', value),
                    support
                )
            end
        end
        index = index + 1;
    end
    return story, knowledge
end

return WhereIsActor
